% SimTAP : Version 1.01 : Last Update Jan 24th 2023 
% Written by Christian Reece
% 
% Reads in input CSV file, grabs parameters, then simulates curves
% CSV File should be formatted using the following parameters :
%
% R_MAT     : Reactivity matrix, first line should be species identifies, the
%             following line should be the reactivity matrix. The reactivity 
%             matrix should be formatted as NSteps x NSpecies. 
%             -1 = consumed in that step +1 = produced in that step
%            *-2 means that species has a stoichiometric value of 2. E.g.
%                         R = [c][c]   use -2
%                         R = 2[c]     use *-2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       EXAMPLE REACTIVITY MATRIX                         %
%                         A + Z* <--> A* --> B                            %
%                                                                         %
%                          A    Z*    A*    B                             %
%                Step 1   -1   -1    +1     0                             %
%                Step 2   +1   +1    -1     0                             %
%                Step 3    0    0    -1    +1                             %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% DIFF_AR   : The diffusivity for Argon in all three zones (cm2/s)
% DIFF_AR_298 : The diffusivity for Argon in all three zones at 298 K (cm2/s)
% MW        : The molecular weights for the species in R_MAT in the same
%             order as R_MAT
% RATE      : The rates for the steps in R_MAT in the same order as in R_MAT
% PREEXP    : Pre-exponential factor, for the steps in R_MAT in the same order as in R_MAT
% EA        : Activation Energy (kj/mol) for the steps in R_MAT in the same order as in R_MAT
%                   Using RATE ignores PREEXP and EA tags.
% TEMP      : Temperature (K) must set to use PREEXP and EA tags
% BED_L     : The bed length (cm). Should be rounded to nearest 0.02cm.
% VOID      : Reactor voidages for all three zones
% BED_R     : Bed radius (cm)
% MCAT      : Catalyst mass (g)
% CSAREA    : Catalytic surface area (cm2/g)
% INIT_COND : The intial condition for each species in the same order as in
%             the first line of R_MAT. Gases in molecules, surface bound
%             species in molecules/cm2
% VERBOSE   : 0 = no output, 1 = shows time in command window, 2 = waitbar
%             higher verbose settings decrease performance of solver, but
%             is a good check to see if solver is converging. -1 does not
%             plot graph/network
% CSV       : 0 = no output, 1 = saves EXIT_COND (exit condtion for all
%             species) and EXIT_dy (exit condition for whole reactor) 
%             as .csv files
% SAMPLES   : Sampling rate for pulses Hz i.e. how many samples per second
% TPSAN     : The data collection time for a pulse (s)
% PULSES    : Number of pulses for TAP experiment, must be set to zero to
%             perform a flow experiment
% FLOWRATE  : Volumetric flow rate (cm3/s), must be set to zero to perform
%             a TAP experiment
% NCONTROL  : 0 = normalisation of ODE solver is turned off
%             1 = normalisation control is on (default). 
%             normalisation speeds up ODE solvers by scaling the RelTol and
%             the AbsTol to the input. However, this can occasionally cause
%             rounding errors which causes species to be created from
%             nothing. Turning it off causes a rougly 3x decrease in
%             simulation speed.
% RESTART   : 0 = (default) uses INIT_COND to run the simulation
%             1 = restart using entire reactor exit condition from the ODE
%             solver using EXIT_dy.csv
% JACOBIAN  : 0 = allow the ODE solver to estimate the jacobian using
%             finite differences.
%             1 = calculate the jacobian pattern before the ODE solver6
%             WARNING: This can cause significant issues
%             if the jacobian pattern is incorrectly generated.
%             Turning this on will provide a 10x speed up for the ODE
%             solver             
% 
% If you're finding that your simulations are taking too long to finish,
% this is alomst certainly down to extremely fast kinetics. The most
% common culprit is a poorly scaled adsorption rate constant. If it is too
% fast the solver will keep decreasing the timestep in order to stop the
% concentration from going negative, which increases computation time!
% Further this only works for 1st and 2nd order reactions, I don't know of
% any 3rd order reactions so let's just forget those for now.
% Clear variables and read in new input file:
clear
% Overwrites waitbar then closes it, otherwise if you keep re-running you
% end up with lots of waitbars.
h = waitbar(0);
close(h);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                REPLACE THIS WITH THE FILENAME OF INTEREST               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
INPUT = readcell('HH_Titration_Coverage_9.csv');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                REPLACE THIS WITH THE FILENAME OF INTEREST               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[xl,yl] = size(INPUT);

% Intialises some variables that are read by the input file
R_MAT_TAG    = 0; % 0 means it has not read R_MAT_TAG yet, 1 means it has.
react_matrix = []; % Reactivity matrix
MW           = []; % Molecular weights
d            = []; % Diffusivity of Argon in each zone (cm2/s)
dscale       = []; % Diffusivity T scaling (0 = off, 1 = on)

r_const      = []; % Rate constants
ea_const     = []; % Activation energies (kj/mol)
A_const      = []; % Pre-exponential factors
bed_L        = []; % Bed zone lengths (cm)


e            = []; % Reactor voidage
rad          = []; % Reactor radius (cm)
mcat         = []; % Mass of catalyst (g)
temp         = []; % Temperature (K)
csarea       = []; % Surface area of catalyst (cm2/g)
init_cond    = []; % Intial condition for simulation (pulse size/coverage)
tmax         = []; % Maximum time for time array.
verbose      = []; % Verbosity of output (0-2).
samples      = []; % Sampling rate (Hz)
pulseno      = []; % Number of pulses in experiment
%%%%%%%%%%%%%%%%%% NOT USED %%%%%%%%%%%%%%%%%%
flowrate     = []; % Flow rate               %
%%%%%%%%%%%%%%%%%% NOT USED %%%%%%%%%%%%%%%%%%
csv_out      = []; % Save INIT_COND output as .csv
ncontrol     = []; % Normalisation control
restreact    = []; % Restart using full reactor conditions
jpatt_gen    = []; % Calculate jacobian pattern

% Loop over every row in the INPUT file
for i = 1:xl
    % Loads in Reactivity Matrix
    if strcmp(INPUT(i,1),'R_MAT')
        if R_MAT_TAG == 0
            % Loop through R_MAT to grab all info
            fprintf('Found R_MAT matrix on line %d\n',i);
            for j = i:i+100
                if strcmp(INPUT{j,1},'R_MAT')
                    % Loop through all species
                    for k = 2:yl
                        
                        % If this is the first line, it should be the
                        % name of the species. If it's not, then the file
                        % is not formatted correctly.
                        if j == i
                            % Grab species names, keep as cell as it's
                            % easier to parse in MATLAB. If the cell is
                            % "missing" it is ignored.
                            if ~ismissing(INPUT{j,k})
                                species(k-1) = INPUT(j,k);
                            end
                        else
                            % Grab the reactivity matrix, use curly
                            % brackets to get number instead of cell. If
                            % the cell is "missing" ignore. Statment is: 
                            % "if not missing" meaning it is filled...
                            if ~ismissing(INPUT{j,k})
                                % If it's just stoichiometric and not 2nd
                                % order reaction need to catch that tag.
                                % The * makes it a character array so
                                % that's how we catch it.
                                if INPUT{j,k} == '*-2'
                                    react_matrix(j-1,k-1) = -22;
                                else
                                    react_matrix(j-1,k-1) = INPUT{j,k};
                                end
                            end
                        end
                    end
                else
                    [numsteps,numspecies] = size(react_matrix);
                    fprintf('Finished reading R_MAT on line %d numsteps : %d numspecies : %d\n',...
                        j,numsteps,numspecies')
                    R_MAT_TAG = 1;
                    break
                end
            end
        end
    end
    
    % Reads in Diffusivity data, always assumes there are 3 zones.
    if strcmp(INPUT(i,1),'DIFF_AR')
        d = [INPUT{i,2},INPUT{i,3},INPUT{i,4}];
        fprintf('Found DIFF_Ar on line %d : ',i)
        fprintf('%.3f %.3f %.3f\n',...
            d(1),d(2),d(3));
        dscale = 0;
    end
    
    % Reads in Diffusivity scaling data, always assumes there are 3 zones.
    if strcmp(INPUT(i,1),'DIFF_AR_298')
        d = [INPUT{i,2},INPUT{i,3},INPUT{i,4}];
        fprintf('Found DIFF_Ar_298 on line %d : ',i)
        fprintf('%.3f %.3f %.3f\n',...
            d(1),d(2),d(3));
        dscale = 1;
    end
    
    % Reads in molecular weights
    if strcmp(INPUT(i,1),'MW')
        fprintf('Found MW on line %d : ',i)
        for j = 2:yl
            % Grabs in the molecular weights of the gases, if the cell is
            % "missing" ignore.
            if ~ismissing(INPUT{i,j})
                MW(j-1) = INPUT{i,j};
                fprintf('%.1f ',...
                    MW(j-1));
            end
        end
        fprintf('\n') % End line after printing.
    end
    
    % Reads in rate constants
    if strcmp(INPUT(i,1),'RATE')
        fprintf('Found RATE on line %d : ',i)
        for j = 2:yl
            % Grabs in the rate constants, if the cell is
            % "missing" ignore.
            if ~ismissing(INPUT{i,j})
                r_const(j-1) = INPUT{i,j};
                fprintf('%.2E ',...
                    r_const(j-1));
            end
        end
        fprintf('\n') % End line after printing.
    end
    
    % Reads in activation energy
    if strcmp(INPUT(i,1),'EA')
        fprintf('Found EA on line %d : ',i)
        for j = 2:yl
            % Grabs in the activation energies, if the cell is
            % "missing" ignore.
            if ~ismissing(INPUT{i,j})
                ea_const(j-1) = INPUT{i,j};
                fprintf('%.2E ',...
                    ea_const(j-1));
            end
        end
        fprintf('\n') % End line after printing.
    end
    
    % Reads in activation energy
    if strcmp(INPUT(i,1),'PREEXP')
        fprintf('Found PREEXP on line %d : ',i)
        for j = 2:yl
            % Grabs in the pre-exponential factors, if the cell is
            % "missing" ignore.
            if ~ismissing(INPUT{i,j})
                A_const(j-1) = INPUT{i,j};
                fprintf('%.2E ',...
                    A_const(j-1));
            end
        end
        fprintf('\n') % End line after printing.
    end
    
    % Reads in bed length, always assumed three zones
    if strcmp(INPUT(i,1),'BED_L')
        bed_L = [INPUT{i,2},INPUT{i,3},INPUT{i,4}];
        fprintf('Found BED_L on line %d : ',i)
        fprintf('%.3f %.3f %.3f\n',...
            bed_L(1),bed_L(2),bed_L(3));
    end
    
    % Reads in reactor voidage, always assumed three zones
    if strcmp(INPUT(i,1),'VOID')
        e = [INPUT{i,2},INPUT{i,3},INPUT{i,4}];
        fprintf('Found VOID on line %d : ',i)
        fprintf('%.3f %.3f %.3f\n',...
            e(1),e(2),e(3));
    end
    
    % Reads in bed radius, always assumes one value
    if strcmp(INPUT(i,1),'BED_R')
        rad = INPUT{i,2};
        fprintf('Found BED_R on line %d : ',i)
        fprintf('%.3f\n',...
            rad);
    end
    
    % Reads in mass of catalyst, always assumes one value
    if strcmp(INPUT(i,1),'MCAT')
        mcat = INPUT{i,2};
        
        fprintf('Found MCAT on line %d : ',i)
        fprintf('%.3f\n',...
            mcat);
    end
    
    % Reads in temperature, always assumes one value
    if strcmp(INPUT(i,1),'TEMP')
        temp = INPUT{i,2};
        
        fprintf('Found TEMP on line %d : ',i)
        fprintf('%.3f\n',...
            temp);
    end
    
    % Reads in catalyst surface area, always assumes one value
    if strcmp(INPUT(i,1),'CSAREA')
        csarea = INPUT{i,2};
        fprintf('Found CSAREA on line %d : ',i)
        fprintf('%.2E\n',...
            csarea);
    end
    
    % Reads in time, always assumes one value
    if strcmp(INPUT(i,1),'TSPAN')
        tmax = INPUT{i,2};
        fprintf('Found TSPAN on line %d : ',i)
        fprintf('%.3f\n',...
            tmax);
    end
    
    % Reads in sample, always assumes one value
    if strcmp(INPUT(i,1),'SAMPLES')
        samples = INPUT{i,2};
        fprintf('Found SAMPLES on line %d : ',i)
        fprintf('%.3f\n',...
            samples);
    end
    
    % Reads in sample, always assumes one value
    if strcmp(INPUT(i,1),'CSV')
        csv_out = INPUT{i,2};
        fprintf('Found CSV on line %d : ',i)
        fprintf('%.3f\n',...
            csv_out);
        fprintf('Will save exit conditions as EXIT_COND.csv and EXIT_dy.csv\n')
    end
    
    % Reads in intial conditions
    if strcmp(INPUT(i,1),'INIT_COND')
        fprintf('Found INIT_COND on line %d : ',i)
        for j = 2:yl
            % Grabs in the molecular weights of the gases, if the cell is
            % "missing" ignore.
            if ~ismissing(INPUT{i,j})
                init_cond(j-1) = INPUT{i,j};
                fprintf('%.3E ',...
                    init_cond(j-1));
            end
        end
        fprintf('\n') % End line after printing.
    end
    
    % Reads in verbosity, always assumes one value
    if strcmp(INPUT(i,1),'VERBOSE')
        verbose = INPUT{i,2};
        fprintf('Found VERBOSE on line %d : ',i)
        fprintf('%.3f\n',...
            verbose);
        if verbose == 2
            fprintf('Using verbose level 2 decreases performance, only use for testing\n')
        end
    end
    % Reads in normalisation control
    if strcmp(INPUT(i,1),'NCONTROL')
        ncontrol = INPUT{i,2};
        fprintf('Found NCONTROL on line %d : ',i)
        fprintf('%.0f\n',...
            ncontrol);
    end
    
    % Reads in pulse numbers, always assumes one value
    if strcmp(INPUT(i,1),'PULSES')
        pulseno = INPUT{i,2};
        fprintf('Found PULSES on line %d : ',i)
        fprintf('%.3f\n',...
            pulseno);
        if pulseno > 1
            fprintf('Running a Pulse Experiment... will feed in exit condition to intial condition\n')
        end
    end
    
    % Reads in flow rate, always assumes one value
    if strcmp(INPUT(i,1),'FLOWRATE')
        pulseno = INPUT{i,2};
        fprintf('Found FLOWRATE on line %d : ',i)
        fprintf('%.3f\n',...
            pulseno);
        if pulseno > 1
            fprintf('Running a Pulse Experiment... will feed in exit condition to intial condition\n')
        end
    end
        
    % Reads in the restart tag, always assumes one value
    if strcmp(INPUT(i,1),'RESTART')
        restreact = INPUT{i,2};
        fprintf('Found RESTART on line %d : ',i)
        fprintf('%.2E\n',...
            restreact);
    end   
        
    % Reads in the jacobian tag, always assumes one value
    if strcmp(INPUT(i,1),'JACOBIAN')
        jpatt_gen = INPUT{i,2};
        fprintf('Found JACOBIAN on line %d : ',i)
        fprintf('%.2E\n',...
            jpatt_gen);
    end  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%% INPUT TAG CHECKING %%%%%%%%%%%%%%%%%%%%%%%%%%%
% Error Checking : Makes sure input file has all the correct tags. Some
% tags will have default values associated with them

% R_MAT tag
if isempty(react_matrix)
    fprintf('Reactivity matrix has not been read correctly, please make sure\n');
    fprintf('that input file is formatted correctly and R_MAT tag has been used\n')
    return
end
% DIFF_Ar tag
if isempty(d)
    fprintf('Diffusivity not read in, please make sure DIFF_Ar tag is used\n')
    return
end
if dscale == 1 && isempty(temp)
    fprintf('Diffusivity T scaling selected but TEMP tag not read in...\n')
    fprintf('Please either use DIFF_AR tag or set the temperature using the TEMP tagd\n')
    return
elseif dscale == 1 && ~isempty(temp)
    fprintf('Diffusivity T scaling selected... scaling diffusivity from 298 to %d K...\n',temp)
    d = (sqrt(temp)./sqrt(298))'.*d; 
end
% MW tag
if isempty(MW)
    fprintf('Molecular weights have not been read in correctly, please make sure MW tag is used\n')
    return
end
% RATE and PREEXP + EA + TEMP tags
if isempty(r_const) && isempty(A_const) && isempy(ea_const)
    fprintf('No kinetic parameters have been read in, please make sure RATE or PREEXP and EA tags are used...\n')
    fprintf('If simulating inert curves, please set RATE to 0 for the first step\n')
    return
elseif isempty(r_const) && ~isempty(ea_const) && ~isempty(A_const) && isempty(temp)
    fprintf('Activation Energies and Pre-Exponentials selected, but no temperature has been set...\n')
    fprintf('Please either use the RATE tag or set the temperature using the TEMP tag\n')
    return
elseif ~isempty(r_const) && ~isempty(ea_const) && ~isempty(A_const) && ~isempty(temp)
    fprintf('Both RATE and PREEXP + EA + TEMP tags have been applied...\n')
    fprintf('Defaulting to using RATE tag\n')
    A_const = [];
    Ea_const = [];
    temp = [];
elseif isempty(r_const) && ~isempty(ea_const) && ~isempty(A_const) && ~isempty(temp)
    fprintf('Activation Energies, Pre-exponential factors, and temperatures have been read in correctly\n')
    % Calculates rate constants
    r_const = A_const.*exp(-ea_const*1000/(8.314*temp));
end
% BED_L tag
if isempty(bed_L) || length(bed_L) ~= 3
    fprintf('Bed lengths have not been read in correctly, please maure sure BED_L tag\n')
    fprintf('is used, or that there are 3 zones in BED_L\n')
    return
end
% VOID tag
if isempty(e) || length(e) ~= 3
    fprintf('Reactor voidages have not been read in correctly, please make sure VOID tag\n')
    fprintf('is used, or that there are 3 zones in VOID\n')
    return
end
% BED_R tag
if isempty(rad)
    fprintf('Reactor bed radius has not been set, please use BED_R tag\n')
    return
end
% MCAT tag
if isempty(mcat)
    fprintf('Catalyst mass has not been read in correctly, please use MCAT tag\n')
    fprintf('If MCAT is not known, just set to 1\n')
    return
end
% CSAREA tag
if isempty(csarea)
    fprintf('Catalyst surface area has not been read in correctly, please use CSAREA tag\n')
    fprintf('If CSAREA is unknown, just set to 1\n')
    return
end
% TSPAN tag
if isempty(tmax)
    fprintf('Max time has not been read in correctly, please use TSPAN tag/n')
    fprintf('Will default to a max time of 2s\n')
    tmax = 2;
end
% SAMPLES tag
if isempty(samples)
    fprintf('Sampling rate has not been read in correctly, please use SAMPLES tag/n')
    fprintf('Will default to a sampling rate of 500 Hz\n')
    samples = 500;
end
% INIT_COND
if isempty(init_cond)
    fprintf('No intial conditions have been read in, please set them using the INIT_COND tag\n')
    return
end
% VERBOSE
if isempty(verbose)
    fprintf('Verbose level has not been set correctly, please use VERBOSE tag\n')
    fprintf('Will be set to default of 2\n')
end
% PULSES
if isempty(pulseno)
    fprintf('PULSES tag has not been set\n')
end
% NCONTROL
if ncontrol == 0
    fprintf('NCONTROL has been set to 0, turning off normalisation control in ODE solver\n')
elseif ncontrol == 1
    fprintf('NCONTROL has been set to 1, turning on normalisation control in ODE solver\n')
elseif isempty(ncontrol)
    fprintf('NCONTROL tag has not been set, setting to default of 1\n')
    fprintf('NCONTROL has been set to 1, turning on normalisation control in ODE solver\n')
    ncontrol = 1;
end
% RESTREACT
if restreact == 0
    fprintf('RESTART has been set to 0, using INIT_COND for intial conditon\n')
elseif restreact == 1
    fprintf('RESTART has been set to 1, using EXIT_dy.csv for intial conditon\n')
    if ~isfile('EXIT_DY.csv')
        fprintf('RESTREACT has been set to 1, but EXIT_dy.csv does not exist...\n')
        fprintf('Are you sure you set the tag correctly?\n')
        fprintf('Exiting Simulation \n')
        return
    end
elseif isempty(restreact)    
    fprintf('RESTART has  not been set, setting to default of 0\n')
    fprintf('RESTART has been set to 0, using INIT_COND for intial conditon\n')
    restreact = 0;
end

% JPATTERN
if jpatt_gen == 0
    fprintf('JACOBIAN has been set to 0, using ODE solver to generate jacobian on the fly\n')
elseif jpatt_gen == 1    
    fprintf('JACOBIAN has been set to 1, calculating jacboian pattern before ODE solver\n')
    fprintf('WARNING: This can cause significant issues if the jacobian pattern is incorrectly generated...\n')
    fprintf('...proceed with caution!\n')
elseif isempty(jpatt_gen)    
    fprintf('JACOBIAN has  not been set, setting to default of 0\n')
    fprintf('JACOBIAN has been set to 0, using ODE solver to generate jacobian on the fly\n')
    jpatt_gen = 0;
end

% FLOW
if isempty(flowrate)
    %fprintf('FLOWRATE tag has not been set... ignoring\n')
end

% FLOW
if isempty(csv_out)
    fprintf('CSV tag has not been set, exit conditions will only be saved to workspace\n')
end

% PULSE vs FLOW checking
if isempty(flowrate) && isempty(pulseno)
    fprintf('Both FLOWRATE tag and PULSES tag have not been set\n')
    fprintf('Please set one in the input file\n')
    return
elseif isempty(flowrate) && ~isempty(pulseno)
    fprintf('*************************************\n')
    fprintf('*    PERFORMING PULSE EXPERIMENT    *\n')
    fprintf('*************************************\n')
elseif isempty(pulseno) && ~ismepty(flowrate)
    fprintf('*************************************\n')
    fprintf('*    PERFORMING FLOW EXPERIMENT     *\n')
    fprintF('*************************************\n')
end

%%%%%%%%%%%%%%%%%%%%%%% END INPUT TAG CHECKING %%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%% ERROR CHECKING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Need to add more error checking - Christian Reece May 4th 2020      %
% Error checking to see if number of rate constants is the same as the
% number of steps
if length(r_const) ~= numsteps
    fprintf('Number of steps (%d) is not equal to number of rate constants (%d)\n',...
        numsteps,length(r_const))
    return
end

% Error checking to see if number of molecular weights is the same as the
% number of species.
if length(MW) ~= numspecies
    fprintf('Number of MW (%d) is not equal to number of species (%d)\n',...
        length(MW),numspecies)
    fprintf('Need to adjust input file\n');
    return
end

% Error checking to see if number of initial conditions is the same as the
% number of species.
if length(init_cond) ~= numspecies
    fprintf('Number of initial conditions (%d) is not equal to number of species (%d)\n',...
        length(init_cond),numspecies)
    fprintf('Need to adjust input file\n');
    return
end

% Error checking to see if the size of the reactivity matrix is consistent
% with the number of species and the number of steps
if ~isequal(size(react_matrix),[numsteps,numspecies])
    fprintf(['Error: Reactivity matrix inconsistent with number of',...
        ' number of species/steps\n'])
    [xl,yl] = size(react_matrix);
    fprintf('React Matrix is size %d x %d, whereas numsteps = %d and numspecies = %d\n',...
        xl,yl,numsteps,numspecies);
    return
end

% Error checking to see if there is a NaN in the diffusivity parameters,
% which usually means the diffusivity for a zone has not been read.
if isnan(sum(d(:)))
    fprintf('Diff_Ar has not been read in correctly with values %.3f %.3f %.3f\n',...
        d(1),d(2),d(3))
    return
end
%%%%%%%%%%%%%%%%%%%%%%%%% END ERROR CHECKING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This loops through and creates edges that go from Reactant --> Step. In
% such a way that the reaction A -> B is drawn as:
% A ---> (1) ---> B
edges = {};
k = 1;
for i = 1:numsteps
    for j = 1:numspecies
        if react_matrix(i,j) > 0
            edges(k,:) = {num2str(i),species{j}};
            k = k+1;
        elseif react_matrix(i,j) < 0
            edges(k,:) = {species{j},num2str(i)};
            k = k+1;
        end
    end
end
        
% Plots the reaction network using MATLABs digraph tool. Uses the edges
% that are calculated in the previous section.
if verbose >= 0
    % Creates a subplot to show network and reactivity data
    subplot(1,8,1:4)
    % Sets the background color to white
    set(gcf,'color','w')
    if ~isempty(edges) % If there is no reactivity, ignore this section
        % Generates a directed graph
        g_network = digraph(edges(:,1),edges(:,2));
        % Grabs the list of the nodes in the edges (for highlighting)
        node_list = table2cell(g_network.Nodes);
        species_locs = zeros(size(node_list));
        % Loops through the node list and creates a logical array of the
        % actual species
        for i = 1:length(node_list)
            for j = 1:length(species)
                if strcmp(string(node_list(i)),string(species(j)))
                    species_locs(i) = 1;
                end
            end
        end    
        species_locs = logical(species_locs);
        % Creates directed graph plot
        g_plot = plot(g_network,'k','LineWidth',1.5);
        % Highlights species
        highlight(g_plot,species_locs,...
            'MarkerSize',8,'NodeLabelColor','red','NodeFontSize',12)
        % Highlights reaction nodes
        highlight(g_plot,~species_locs,...
            'NodeLabelColor','red','NodeFontSize',10)
        % General plot fixing - to make it look nicer
        layout(g_plot,'force')
        ax = gca;
        ax.Position = [0,0,0.5,1.0];
        g_plot.ArrowSize = 10;
        g_plot.ArrowPosition = 0.45;
        g_plot.NodeFontWeight = 'bold';
    else
        fprintf('No chemistry detected, not drawing reaction network\n')
    end
    box off
    set(gca,'visible','off')
    title('Reaction Network')
    
    % Sets the time array
    tspan = linspace(0,tmax,tmax*samples);
    
    % This runs the TAP Pulse simulation
    tstart = tic;
    [psim,M0,C0,dy] = SimTAP_Init(MW,d,react_matrix,bed_L,e,rad,tspan,mcat,...
        csarea,r_const,init_cond,verbose,pulseno,species,csv_out,tstart,...
        ncontrol,restreact,jpatt_gen);
    tend = toc(tstart);
    
    fprintf('Simulation took %.1f seconds to complete\n',tend)
    
    % Plots the calculated exit flux from the reactor simulation.
    subplot(1,6,4:6)
    hold off
    % Find which species is a gas
    g_loc = find(MW>0);
    
    if pulseno == 1
        % Plots the gases
        plot(tspan,psim(:,g_loc,end),'LineWidth',1.5)
        legend(species(g_loc))
        xlabel('Time')
        ylabel('Exit Flux mols/s')
        title('Pulses')
        drawnow
    else
        % If it's a multipulse experiment, plots the integrated responses
        plot(M0(g_loc,:)','o')
        legend(species(g_loc))
        xlabel('Pulses')
        ylabel('Zeroth Moment/s')
        title(['Pulses : ',num2str(pulseno)])
        drawnow
    end
else
    % Sets the time array
    tspan = linspace(0,tmax,tmax*samples);
    
    tstart = tic;
    [psim,M0,C0,dy] = SimTAP(MW,d,react_matrix,bed_L,e,rad,tspan,mcat,...
        csarea,r_const,init_cond,verbose,pulseno,tstart);
    tend = toc(tstart);
    
    fprintf('Simulation took %.1f seconds to complete\n',tend)
end