% SimTAP_Init : Version 1.3 : Last Update Jan 23rd 2023 
% Written by Christian Reece
%
% This sets up the ODE solver using the parameters that are read in from
% the INPUT file by SimTAP. See SimTAP for a list of parameters
% that can be set by the simulation. The required parameters are:
% MW           : The molecular weights of the species, a MW of 0 indicates
%                that the species is surface bound
% d            : The diffusivites of Argon through each zone (cm2/s)
% react_matrix : The reactivity matrix (Nsteps x Nspecies)
% bed_L        : The lengths of the zones (cm)
% e            : The voidage of the zones
% rad          : Reactor radius (cm)
% tpsan        : The time array (s)
% mcat         : The mass of catalyst (g)
% csarea       : The catalytic surface area (cm2/g)
% r_const      : Array of rate constants (Nsteps)
% init_cond    : The inital conditions for the simulation for each species
%                in the reactivity matrix. Pulse sizes / coverages.
% verbose      : 0=no output, 1=shows time in command window, 2=waitbar
%                higher verbose settings decrease performance of solver,
%                but is a good check to see if solver is converging.
% pulseno      : 1 = single pulse experiment, things are pretty simple...
%                >1 = multipulse experiment so it feeds in the exit
%                condition as the intial condition and re-runs the solver
% species      : Used for plotting in a multipulse experiment with
%                verbose > 0
% ncontrol     : Controls normalisation control for ODE solver solution

function [psim,M0,C0,dy] = SimTAP_Init(MW,d,react_matrix,bed_L,e,rad,tspan,mcat,...
    csarea,r_const,init_cond,verbose,pulseno,species,csv_out,tstart,...
    ncontrol,restreact,jpatt_gen)
psim = []; M0 = []; C0 = []; dy = [];

% Setting the Physical Parameters
MW(MW==0)       = NaN;                  % Sets surface species to NaN
issurf          = isnan(MW);            % Is the species surface bound?
numspecies        = length(MW);           % Works out how many gases.


% Calculates diffusion coefficients for each zone and gas.
D               = (sqrt(40)./sqrt(MW))'.*d;    % Diffusion constants

% Setting up points
grid            = round(cumsum(bed_L*50)); % Last grid point location for each zone
grid_points     = grid(3);              % Number of grid points for gaseous species
catal_points    = grid(2)-grid(1);      % Number of points in catalyst zone
dx              = sum(bed_L)/grid_points; % Distance (cm) per grid point

% Physical Parameters of Catalyst + Pulse
sarea     = csarea*mcat;                 % Total surface area of active sites

react_matrix = react_matrix';           % Rotates the array to make the
% programming much easier
% Rate constants array
numsteps        = length(r_const);      % Number of kinetic steps

% Calculating grid locations
for i = 1:numspecies
    G(i,:) = [grid_points*(i-1)+1,grid+grid_points*(i-1)];
end

% Works out kinetics, this will find out the grid locations for each
% kinetic step for each mass.
stoic_cell = {};
react_cell = {};
scale_cell = [];
for i = 1:numspecies
    for j = 1:numsteps
        % Is the gas being consumed or produced (-1 consumed, +1 produced)?
        consumed = react_matrix(i,j);
        % Skips this loop if the species is not being consumed or produced in
        % this kinetic step.
        if consumed ~= 0
            for k = 1:numspecies
                % See what speceies are being consumed
                if react_matrix(k,j) == -1
                    % Sets marker saying this species is going to be used for the
                    % kinetic equation
                    % R = [c]
                    loc(k) = 1;
                elseif react_matrix(k,j) == -2
                    % The reaction is a second order reaction consuming
                    % itself
                    % R = [c][c]
                    loc(k) = 2;
                elseif react_matrix(k,j) == -22
                    % The reaction consumes two sites but is not dependent
                    % on the concentration of the species squared
                    % R = 2[c]
                    loc(k) = 1;
                elseif react_matrix(k,j) < -2 && react_matrix(k,j) ~= -22
                    loc(k) = 1;
                else
                    loc(k) = 0;
                end
            end
            % Works out the kinetic equations. Need to reset variables
            % arrayloc and need_scale on each loop.
            arrayloc = [];
            need_scale = 0;
            reactwith = [];
            for L = 1:numspecies
                % If species i is being consumed by species L.
                if loc(L) == 1
                    arrayloc = [arrayloc,(G(L,2):G(L,3)-1)'];
                    % Lists which gases are reacting with what.
                    reactwith = [reactwith,L];
                elseif loc(L) == 2
                    % If is a second order consumption of a species
                    arrayloc = [arrayloc,(G(L,2):G(L,3)-1)',(G(L,2):G(L,3)-1)'];
                    reactwith = [reactwith,L];
                end
                % Is it a gas to a surface reaction step?
                if issurf(i) == 0 && issurf(L) == 1
                    need_scale = 1;
                end
            end
            % Combines them into the cell arrays that SimTAP_ODE uses.
            stoic_cell(i,j) = {arrayloc};
            scale_cell(i,j) = need_scale;
            react_cell(i,j) = {reactwith};
        end
    end
end

assignin('base','stoic_cell',stoic_cell);
assignin('base','react_cell',react_cell);
assignin('base','scale_cell',scale_cell);

% Physical Parameters of the bed
vr = bed_L.*rad.^2*pi*e(2); % Void volume of the reactor (cm3)
vc = vr(2)*(1-e(2));        % Volume of catalyst in catalyst zone (cm3)
dv = vr.*e./[grid(1),grid(2)-grid(1)-1,grid(3)-grid(2)]; % Void volume per grid point in each zone
Sv = (sarea/vc)*(1-e(2))/e(2); % Converts to apparent rate constant. For some reason we need a second e value
fprintf('k -> kapp scaling constant is : %1.3E\n',Sv);
Sv = Sv/e(2);                  % to get the solutions to align with the analytical solutions?
                               % But wait is this correct? What's going on?

% Derirative Setup : 1 Gas
dy = zeros(max(G(:)),1);                    % y variable set to 0

% Sets up the intial conditions for the simulation (pulse/coverage)
for i = 1:length(init_cond)
    if init_cond(i) > 0 % If the species is being initliased
        if issurf(i) == 1 % If the speicies is surface bound
            dy(G(i,2):G(i,3)-1) = init_cond(i);
        else           % If the species is a gas, mols to mols/cm3
            dy(G(i,1)+1) = init_cond(i)/dv(1);
        end
    end
end


dy2 = zeros(size(dy));                      % Initialise derirative

% For plotting and updates, opens the waitbar and brings it to the front
tmax = max(tspan);
% Only use waitbar if verbose == 2
if verbose == 2
    % Forces waitbar to the front, but makes it "modal" meaning you cannnot
    % click outside of waitbar
    wbar = waitbar(0,'Time = 0','WindowStyle','modal');
    % This sets it back to noraml style so you can click off it.
    set(wbar,'WindowStyle','normal')
else
    wbar = [];
end
assignin('base','G',G);

if jpatt_gen == 1
    % Generate Jacobian Sparse Matrix Pattern
    % We do this by running the SimTAP_ODE function with dummy
    % variables, this should be sufficient to identify the Jpattern
    % matrix for diffusion and first order reactions
    jpattern_test = zeros(size(dy));
    for i = 1:length(dy)
        dy_dummy = zeros(size(dy));
        dy_dummy(i) = 1;
        dy2_dummy = SimTAP_ODE([],dy_dummy,dy2,D,e,dx,G,numspecies,numsteps,...
            react_matrix,stoic_cell,r_const,scale_cell,Sv,tmax,wbar,verbose);
        locs = find(dy2_dummy);
        jpattern_test(locs,i) = 1;
    end
    % Then we loop through the grid that represents the chemistry, we
    % use the "stoic cell" to identify the number of steps + the number
    % of species, then we loop through the total number of species and
    % see what they react with.
    [num_spec,num_step] = size(stoic_cell);
    for i = 1:num_spec
        % Find grid space locations for each species
        for j = G(i,2):G(i,3)-1
            % A counter that starts at 1 for the grid spacing locations
            m = j-G(i,2)+1;
            for l = 1:num_step
                % Identify what that species is reacting with for a
                % given step
                locs = stoic_cell{i,l};
                % If it's empty then ignore it otherwise add it to the
                % jacobian sparse matrix.
                if ~isempty(locs)
                    jpattern_test(j,locs(m,:)) = 1;
                end
            end
        end
    end
    % Turn the Jpattern on/off
    opt_1 = odeset('Jpattern',jpattern_test);
    assignin('base','jpattern_test',jpattern_test);
else
    opt_1 = odeset('Jpattern',[]);
end

% Turn normalisation control on/off
if ncontrol == 1
    opt_2 = odeset('Stats','off','RelTol',1E-7,'AbsTol',1E-9,...
        'NormControl','on','NonNegative',1:length(dy));
elseif ncontrol == 0
    opt_2 = odeset('Stats','off','RelTol',1E-7,'AbsTol',1E-9,...
        'NormControl','off','NonNegative',1:length(dy));
end

% Combine options structures.
options = odeset(opt_1,opt_2);

for I = 1:pulseno
    if I == 1
        if restreact == 1
            if isfile('EXIT_dy.csv')
                fprintf('Input condition has been read from EXIT_dy.csv\n')
                dy = readmatrix('EXIT_dy.csv'); % Reads in from the EXIT_dy.csv file.
                % Sets up the intial conditions for the simulation (pulse/coverage)
                for i = 1:length(init_cond)
                    if issurf(i) == 1 % If the speicies is surface bound?
                        % Skip this!
                    else
                        if init_cond(i) > 0 % Assumes gas is being repeatedly pulsed in
                            % If the species is a gas, mols to mols/cm3
                            dy(G(i,1)+1) = dy(G(i,1)+1)+init_cond(i)/dv(1);
                        end
                    end
                end
            else
                fprintf('RESTREACT has been set to 1, but EXIT_dy.csv does not exist...\n')
                fprintf('Are you sure you set the tag correctly?\n')
                fprintf('Exiting Simulation \n')
                return
            end
        end
        
        % ODE15s solver
        [tspan,dy] = ode15s(@(t,dy,cnt)...
            SimTAP_ODE(t,dy,dy2,D,e,dx,G,numspecies,numsteps,...
            react_matrix,stoic_cell,r_const,scale_cell,Sv,tmax,wbar,verbose),...
            tspan,dy,options);
        
        % Flip derirative matrix so it's the right way round
        dy = dy';
        
        % Calculates flux out of reactor exit F = C*D/eb/dx/dx*dv
        % C (mol/cm3)*(cm2/s)/(cm*cm)*(cm3) = mols/s
        for i = 1:numspecies
            if issurf(i) == 0
                psim(:,i,I) = dy(G(i,4),:)*dv(3)*(D(i,3)./e(3))./(dx*dx);
                M0(i,I)     = trapz(tspan,psim(:,i));
            else
                C0(i,I) = mean(dy(G(i,2):G(i,3)-1,end));
            end
        end
        
        
        allspecies  = zeros(I,numspecies);
        isgas_T       = find(MW>0);            % Find gases
        issurf_T      = isnan(MW);             % Find surface species
        allspecies(:,isgas_T) = M0(isgas_T,:)';
        allspecies(:,issurf_T) = C0(issurf_T,:)';
        Table_EXIT  = [species;num2cell(allspecies)];
        
        % If CSV tag is set output exit conditions as a .csv file, and also
        % output the pulses as a .txt file
        if csv_out == 1
            writecell(Table_EXIT,'EXIT_COND.csv');
            writematrix(dy(:,end),'EXIT_dy.csv');
            % Loop over all species
            for i = 1:numspecies
                % If species is a gas
                if issurf(i) == 0
                    % Grabs the peak
                    ydata = psim(:,i,:);
                    xdata = repmat(tspan',1,I);
                    fid = fopen([species{i},'.txt'],'w');
                    fprintf(fid,'%.4E %.4E\n',[xdata(:),ydata(:)]');
                    fclose(fid);
                end
            end
        end
        
        assignin('base','Table_EXIT',Table_EXIT);
        
        
        dy_prev = dy;
        runtime = toc(tstart);
        fprintf('Pulse Number : %d / %d completed at %.1f seconds\n',I,pulseno,runtime)
        fprintf('EXIT COND : ')
        fprintf('%1.3E ',allspecies)
        fprintf('\n')
    elseif I > 1
        % Resets intial condition to the final condition from previous sim
        dy = dy_prev(:,end);
        % Sets up the intial conditions for the simulation (pulse/coverage)
        for i = 1:length(init_cond)
            if issurf(i) == 1 % If the speicies is surface bound?
                % Skip this!
            else
                if init_cond(i) > 0 % Assumes gas is being repeatedly pulsed in
                    % If the species is a gas, mols to mols/cm3
                    dy(G(i,1)+1) = dy(G(i,1)+1)+init_cond(i)/dv(1);
                end
            end
        end
        
        assignin('base','dy_prev',dy_prev);
        
        % ODE15s solver
        [tspan,dy] = ode15s(@(t,dy,cnt)...
            SimTAP_ODE(t,dy,dy2,D,e,dx,G,numspecies,numsteps,...
            react_matrix,stoic_cell,r_const,scale_cell,Sv,tmax,wbar,verbose),...
            tspan,dy,options);
        
        % Flip derirative matrix so it's the right way round
        dy = dy';
        
        % Calculates flux out of reactor exit F = C*D/eb/dx/dx*dv
        % C (mol/cm3)*(cm2/s)/(cm*cm)*(cm3) = mols/s
        for i = 1:numspecies
            if issurf(i) == 0
                psim(:,i,I) = dy(G(i,4),:)*dv(3)*(D(i,3)./e(3))./(dx*dx);
                M0(i,I)     = trapz(tspan,psim(:,i,I));
            else
                C0(i,I) = mean(dy(G(i,2):G(i,3)-1,end));
            end
        end
        
        
        allspecies  = zeros(I,numspecies);
        isgas_T       = find(MW>0);            % Find gases
        issurf_T      = isnan(MW);           % Find surface species
        allspecies(:,isgas_T) = M0(isgas_T,:)';
        allspecies(:,issurf_T) = C0(issurf_T,:)';
        Table_EXIT  = [species;num2cell(allspecies)];
        
        % If CSV tag is set output exit conditions as a .csv file, and also
        % output the pulses as a .txt file
        if csv_out == 1
            writecell(Table_EXIT,'EXIT_COND.csv');
            writematrix(dy(:,end),'EXIT_dy.csv');
            % Loop over all species
            for i = 1:numspecies
                % If species is a gas
                if issurf(i) == 0
                    % Grabs the peak
                    ydata = squeeze(psim(:,i,:));
                    xdata = repmat(tspan',1,I);
                    fid = fopen([species{i},'.txt'],'w');
                    fprintf(fid,'%.4E %.4E\n',[xdata(:),ydata(:)]');
                    fclose(fid);
                end
            end
        end
        
        assignin('base','Table_EXIT',Table_EXIT);
        
        if verbose >= 0
            % Plots the calculated exit flux from the reactor simulation.
            subplot(1,6,4:6)
            hold off
            % Find which species is a gas
            loc = find(MW>0);
            % Plots the gases
            plot(tspan,psim(:,loc,end),'LineWidth',1.5)
            legend(species(loc))
            xlabel('Time')
            ylabel('Exit Flux mols/s')
            title(['Pulse Number : ',num2str(I),' / ',num2str(pulseno)])
            drawnow
        end
        
        
        dy_prev = dy;
        runtime = toc(tstart);
        fprintf('Pulse Number : %d / %d completed at %.1f seconds\n',I,pulseno,runtime)        
        fprintf('EXIT COND : ')
        fprintf('%1.3E ',allspecies(I,:))
        fprintf('\n')
    end
end
