
% SimTAP_ODE : Version 1.4 : Last Update 27th May 2020
% Written by Christian Reece
% 
% This is the function that is sent to the ODE15s solver. It solves the
% diffusion reaction equation for each species involved in the simulation.
% This function is what controls the computation time. Only necessary
% operations should be in this function, any unnecessary functions should
% be offloaded to SimTAP and fed in as parameters.
% The required parameters are:
% t            : The time from the ODE solver, not set by user
% dy           : The matrix of concentrations (bed position x time), this
%                is constantly updated by the ODE solver, but is initally
%                fed in as the intial condition of the simulation
% dy2          : The derirative of dy as calculated in the function,
%                this is intially fed in as zeros
% D            : Matrix of diffusivites nspecies x nzones (cm2/s)
% e            : The voidage of the zones
% dx           : Step length along the bed (cm)
% G            : Matrix of grid locations for each of the zones and each of
%                the gases this is calculated by SimTAP.
% numspecies   : Number of species to be simulated
% numsteps     : Number of steps
% react_matrix : The reactivity matrix from the INPUT file (see SimTAPMain)
% stoic_cell   : The matrix cell that contains information on the
%                stoichiometry of the reactions. Tells the solver which
%                array locations to use for the kinetics.
% r_const      : Array of rate constants (Nsteps)
% scale_cell   : Contains information on whether or that kinetic step is
%                from a gas to a surface. If it is, it scales using the
%                surface to gas scaling coefficient
% Sv           : Surface to gas scaling coefficient
% tmax         : Maximum simulation time (s), used for verbose
% wbar         : Waitbar for verbose = 2
% verbose      : 0=no output, 1=shows time in command window, 2=waitbar
%                higher verbose settings decrease performance of solver,
%                but is a good check to see if solver is converging.

function dy2 = SimTAP_ODE(t,dy,dy2,D,e,dx,G,numspecies,numsteps,...
    react_matrix,stoic_cell,r_const,scale_cell,Sv,tmax,wbar,verbose)

for i = 1:numspecies
    % Works out diffusional component, if the MW is NaN assume it's a
    % surface and do not calculate diffusion
    if ~isnan(D(i,1))
        dy2 = Diffusion_Solver(dy,dy2,G(i,:),D(i,:),e,dx);
    end
    % Calculates kinetics
    for j = 1:numsteps
        % Unpacks the array locations for the kinetic multiplcation
        % If there is no chemistry skip this part
        if ~isempty(stoic_cell)
            arrayloc = stoic_cell{i,j};
        else
            arrayloc = [];
        end
        % Works out the reaction order (1st, 2nd, no 3rd order
        % reactions can be simulated yet... need to expand?)
        [~,order] = size(arrayloc);
        if order == 1
            % If first order, only have to mulitply by one species...
            % Stoic matrix is there to work out if the species is being
            % consumed or produced. If the species if first order but
            % comsumes two sites need to add that capability using -22 tag.
            if react_matrix(i,j) == -22
                dy2(G(i,2):G(i,3)-1) = dy2(G(i,2):G(i,3)-1) +...
                    dy(arrayloc)*r_const(j)*(-2)*...
                    (Sv^scale_cell(i,j));
            else
                dy2(G(i,2):G(i,3)-1) = dy2(G(i,2):G(i,3)-1) +...
                    dy(arrayloc)*r_const(j)*react_matrix(i,j)*...
                    (Sv^scale_cell(i,j));
            end
        elseif order == 2
            % If second order, need to multiply by both species...
            % Stoic matrix is there to work out if the species is being
            % consumed or produced.
            
            % If the species is second order but comsumes two sites need to 
            % add that capability using -22 tag.
            if react_matrix(i,j) == -22
                dy2(G(i,2):G(i,3)-1) = dy2(G(i,2):G(i,3)-1) +...
                dy(arrayloc(:,1)).*dy(arrayloc(:,2))*r_const(j)*(-2)*...
                (Sv^scale_cell(i,j));
            % All other second order reactions
            else
                dy2(G(i,2):G(i,3)-1) = dy2(G(i,2):G(i,3)-1) +...
                (dy(arrayloc(:,1)).*dy(arrayloc(:,2)).*r_const(j).*(react_matrix(i,j)).*...
                (Sv^scale_cell(i,j)));
            end
        end
    end
end


% Updates waitbar... at the moment this is very slow and has lots of
% overhead. Increases simulation time by about 400%
if verbose == 2
    waitbar(t/tmax,wbar,sprintf('Solver Time = %.2E',t))
end
% This is much faster, but still has an overhead. Increases simulation time
% by about 25%
if verbose == 1
    fprintf('time = %.3E\n',t);
end